from datetime import date
from typing import Optional, Union

from FinanceDataReader import DataReader as _DataReader
from pandas import DataFrame
from pandas import concat as _concat


def data_timeseries(
    ticker: str,
    /,
    *,
    sdate: Optional[Union[date, str]] = None,
    edate: Optional[Union[date, str]] = None,
) -> DataFrame:
    return _DataReader(ticker, sdate, edate, None, None)


def data_fred_timeseries(
    ticker: str,
    /,
    *,
    sdate: Optional[Union[date, str]] = None,
    edate: Optional[Union[date, str]] = None,
) -> DataFrame:
    """FRED: Federal Reserve Economic Data"""
    return _DataReader(ticker, sdate, edate, None, "FRED")


def data_krx_timeseries(
    ticker: str,
    /,
    *,
    sdate: Optional[Union[date, str]] = None,
    edate: Optional[Union[date, str]] = None,
) -> DataFrame:
    """KRX: Korea Exchange"""
    return _DataReader(ticker, sdate, edate, "KRX", None)


def data_sse_timeseries(
    ticker: str,
    /,
    *,
    sdate: Optional[Union[date, str]] = None,
    edate: Optional[Union[date, str]] = None,
) -> DataFrame:
    """SSE: Shanghai Stock Exchange"""
    return _DataReader(ticker, sdate, edate, "SSE", None)


def data_szse_timeseries(
    ticker: str,
    /,
    *,
    sdate: Optional[Union[date, str]] = None,
    edate: Optional[Union[date, str]] = None,
) -> DataFrame:
    """SZSE: Shenzhen Stock Exchange"""
    return _DataReader(ticker, sdate, edate, "SZSE", None)


def data_hkex_timeseries(
    ticker: str,
    /,
    *,
    sdate: Optional[Union[date, str]] = None,
    edate: Optional[Union[date, str]] = None,
) -> DataFrame:
    """HKEX: Hong Kong Exchanges"""
    return _DataReader(ticker, sdate, edate, "HKEX", None)


def data_tse_timeseries(
    ticker: str,
    /,
    *,
    sdate: Optional[Union[date, str]] = None,
    edate: Optional[Union[date, str]] = None,
) -> DataFrame:
    """TSE: Tokyo Stock Exchange"""
    return _DataReader(ticker, sdate, edate, "TSE", None)


def data_hose_timeseries(
    ticker: str,
    /,
    *,
    sdate: Optional[Union[date, str]] = None,
    edate: Optional[Union[date, str]] = None,
) -> DataFrame:
    """HOSE: Ho Chi Minh Stock Exchange"""
    return _DataReader(ticker, sdate, edate, "HOSE", None)


def data_timeseries_tickers(
    tickers: list[str],
    /,
    *,
    sdate: Optional[Union[date, str]] = None,
    edate: Optional[Union[date, str]] = None,
    exchange: Optional[str] = None,
) -> DataFrame:
    df = DataFrame()
    for ticker in dict.fromkeys(tickers):
        dfi = _DataReader(ticker, sdate, edate, exchange, None)
        if isinstance(dfi, DataFrame):
            dfi = dfi[["Close"]]
            dfi.columns = [ticker]
            df = _concat([df, dfi], axis=1)
    return df
