import typer

from tompy.data import data_timeseries
from tompy.dataframe import dataframe_read_csv
from tompy.series import series_price_metrics

app = typer.Typer()


@app.command()
def timeseries(savefile: str, ticker: str) -> None:
    """
    Save timeseries
    """
    df = data_timeseries(ticker)
    df = df.dropna()
    df.to_csv(savefile)


@app.command()
def metrics(
    readfile: str, col: str, *, ann_factor: int = 252, metric_mode: int = 2
) -> None:
    """
    Calculate price timeseries metrics
    """
    df = dataframe_read_csv(readfile)
    df = df.dropna()
    m = series_price_metrics(df[col], ann_factor, mode=metric_mode)
    print(m)


if __name__ == "__main__":
    app()
